CookyChat = LibStub("AceAddon-3.0"):NewAddon("CookyChat","AceConsole-3.0","AceEvent-3.0")
CookyChatHandler = CreateFrame("Frame")
-- |cffFF69B4 pink
-- |cffBF5FFF purple

local icon = LibStub("LibDBIcon-1.0", true)
local ldb = LibStub:GetLibrary("LibDataBroker-1.1")
local cookyToolTip = "|cffBF5FFFCooky|rChat\n|cffFF69B4Left-Click|r: toggle on/off.\n|cffFF69B4Shift Left-Click|r: toggle \"Semi-Off\" mode on/off.\n|cffFF69B4Right-Click|r: open settings menu."

local CookyChatLBD = ldb:NewDataObject("CookyChat", {
	label = "CookyChat",
	type = "data source",
	text = "CookyChat",
	icon = "Interface\\AddOns\\CookyChat\\Media\\icon",
})

function CookyChat:OnInitialize()
	CookyChat.db = LibStub("AceDB-3.0"):New("CookyChatDB", CookyChat.defaults, true)
	
	LibStub("AceConfig-3.0"):RegisterOptionsTable("CookyChat", self.options)
	self.Groups = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("CookyChat", "|cffBF5FFFCooky|rChat", nil, "groups")
	
	icon:Register("CookyChat", CookyChatLBD, self.db.profile.icon)
	
	CookyChatHandler:RegisterEvent("CHAT_MSG_BN_WHISPER")
	CookyChatHandler:RegisterEvent("CHAT_MSG_SAY")
	CookyChatHandler:RegisterEvent("CHAT_MSG_GUILD")
	CookyChatHandler:RegisterEvent("CHAT_MSG_OFFICER")
	CookyChatHandler:RegisterEvent("CHAT_MSG_PARTY")
	CookyChatHandler:RegisterEvent("CHAT_MSG_PARTY_LEADER")
	CookyChatHandler:RegisterEvent("CHAT_MSG_RAID")
	CookyChatHandler:RegisterEvent("CHAT_MSG_RAID_LEADER")
	CookyChatHandler:RegisterEvent("CHAT_MSG_RAID_WARNING")
	CookyChatHandler:RegisterEvent("CHAT_MSG_WHISPER")
	CookyChatHandler:RegisterEvent("CHAT_MSG_YELL")
	
	CookyChat:CreateGroups()
	
	if self.db.profile.semiOnOff then
		CookyChatLBD.icon = "Interface\\AddOns\\CookyChat\\Media\\icon_r"
	elseif self.db.profile.globalActive then
		CookyChatLBD.icon = "Interface\\AddOns\\CookyChat\\Media\\icon"
	else
		CookyChatLBD.icon = "Interface\\AddOns\\CookyChat\\Media\\icon_bw"
	end
	
	if type(self.db.profile.blacklist) == "table" then
		self.db.profile.blacklist = nil
		self.db.profile.numBlackList = nil
	end
end

SlashCmdList.COOKYCHATSLASH = function() InterfaceOptionsFrame_OpenToCategory(CookyChat.Groups) end
SLASH_COOKYCHATSLASH1 = '/cooky'
SLASH_COOKYCHATSLASH2 = '/cookychat'

function CookyChatLBD:OnEnter()
	GameTooltip:SetOwner(self, "ANCHOR_NONE")
	GameTooltip:SetPoint("TOPLEFT", self, "BOTTOMLEFT")
	GameTooltip:ClearLines()
	GameTooltip:AddLine(cookyToolTip, 1, 1, 1, 1)
	GameTooltip:Show()
end

function CookyChatLBD:OnClick(button)
	if button == "LeftButton" and IsShiftKeyDown() then
		print("|cffBF5FFFCooky|rChat: Semi-Off mode |cffFF69B4"..CookyChat:SemiOnOff(true))
		CookyChat:UpdateDisplay()
	elseif button == "LeftButton" then
		print("|cffBF5FFFCooky|rChat: |cffFF69B4"..CookyChat:OnOrOff(true))
		CookyChat:UpdateDisplay()
	elseif button == "RightButton" then
		InterfaceOptionsFrame_OpenToCategory(CookyChat.Groups)
	end
end

function CookyChat:SemiOnOff(toggle)
	if toggle then
		self.db.profile.semiOnOff = not self.db.profile.semiOnOff
	end

	if self.db.profile.semiOnOff then 
		return "On."
	else 
		return "Off." 
	end
end

function CookyChat:OnOrOff(toggle)
	if toggle then
		self.db.profile.globalActive = not self.db.profile.globalActive
	end

	if self.db.profile.globalActive then 
		return "On."
	else 
		return "Off." 
	end
end

function CookyChat:RefreshMMButton()
	if icon then
		icon:Refresh("CookyChat", self.db.profile.icon)
		if self.db.profile.icon.hide then
			icon:Hide("CookyChat")
		else
			icon:Show("CookyChat")
		end
	end
end

function CookyChat:UpdateDisplay()
	if self.db.profile.semiOnOff then
		CookyChatLBD.icon = "Interface\\AddOns\\CookyChat\\Media\\icon_r"
	elseif self.db.profile.globalActive then
		CookyChatLBD.icon = "Interface\\AddOns\\CookyChat\\Media\\icon"
	else
		CookyChatLBD.icon = "Interface\\AddOns\\CookyChat\\Media\\icon_bw"
	end
end

function CookyChat:OnBlacklist(event, shortSource)
	if self.db.profile.blacklist ~= "" and event ~= "CHAT_MSG_BN_WHISPER" then
		local blacklistTbl = {}
		for v in string.gmatch(self.db.profile.blacklist, "[^ ]+") do
			tinsert(blacklistTbl, v)
		end
	
		for i = 1, #blacklistTbl do
			if string.lower(blacklistTbl[i]) == string.lower(shortSource) then
				return true
			end
		end
		
		return false
	end
	
	return false
end

function CookyChat:OnWhitelist(event, shortSource)
	semiByPass = false
	if self.db.profile.globalWhitelist ~= "" and event ~= "CHAT_MSG_BN_WHISPER" then
		local whitelistTbl = {}
		for v in string.gmatch(self.db.profile.globalWhitelist, "[^ ]+") do
			tinsert(whitelistTbl, v)
		end
	
		for i = 1, #whitelistTbl do
			if string.lower(whitelistTbl[i]) == string.lower(shortSource) then
				if self.db.profile.semiOnOff then
					semiByPass = true
				end
				return true
			end
		end
		
		return false
	end
	
	return false
end

function CookyChat:OnGroupWhitelist(event, num, shortSource)
	if not self.db.profile.whitelist[num] or self.db.profile.whitelist[num] == "" or event == "CHAT_MSG_BN_WHISPER" then
		return true
	end
	
	local whitelistTbl = {}
	for v in string.gmatch(self.db.profile.whitelist[num], "[^ ]+") do
		tinsert(whitelistTbl, v)
	end
	
	for i = 1, #whitelistTbl do
		if string.lower(shortSource) == string.lower(whitelistTbl[i]) then
			return true
		end
	end
	
	return false
end

local compareMessage
local trigger
local spamBlock = false
local msgNumber = 0

function CookyChat:ExactCheck(compareMessage, trigger)
	local checkTbl = {}
	
	for v in string.gmatch(compareMessage, "[^ ]+") do
		tinsert(checkTbl, v)
	end
	
	for i = 1, #checkTbl do
		if trigger == checkTbl[i] then
			return true
		end
	end
	
	return false
end

function CookyChat:FindTrigger(message, event, shortSource, fullSource, pID)
	for i = 1, self.db.profile.numFilled do
		if not self.db.profile.groupCaseSensitive[i] then 
			compareMessage = string.lower(message)
			trigger = string.lower(self.db.profile.trigger[i])
		else
			compareMessage = message
			trigger = self.db.profile.trigger[i]
		end
		
		if self.db.profile.exactMode and (semiByPass or self.db.profile.globalActive) and self.db.profile.groupActive[i] and not spamBlock then
			if CookyChat:ExactCheck(compareMessage, trigger) then
				C_Timer.After(10, function() msgNumber = 0 spamBlock = false end)
				msgNumber = msgNumber + 1
				if msgNumber == self.db.profile.spamAmount then
					spamBlock = true
				end
				
				if (semiByPass or CookyChat:OnGroupWhitelist(event, i, shortSource)) then
					CookyChat:Reply(event, i, shortSource, fullSource, pID)
				end
			end
			
		elseif string.find(compareMessage, trigger) and (semiByPass or self.db.profile.globalActive) and self.db.profile.groupActive[i] and not spamBlock then
			C_Timer.After(10, function() msgNumber = 0 spamBlock = false end)
			msgNumber = msgNumber + 1
			if msgNumber == self.db.profile.spamAmount then
				spamBlock = true
			end
			
			if (semiByPass or CookyChat:OnGroupWhitelist(event, i, shortSource)) then
				CookyChat:Reply(event, i, shortSource, fullSource, pID)
			end
		end
	end		
end

function CookyChat:Reply(event, num, shortSource, fullSource, pID)
	if event == "CHAT_MSG_SAY" and shortSource ~= UnitName("player") then -- If message is from say and source isn't player
		if self.db.profile.seperateChannelActivityOption then -- If seperate channel per group and group is active
			if self.db.profile.groupChannelActive[num][1] then
				SendChatMessage(self.db.profile.reply[num],"SAY")
			end
		else
			SendChatMessage(self.db.profile.reply[num],"SAY")
		end
	elseif event == "CHAT_MSG_YELL" and shortSource ~= UnitName("player") then
		if self.db.profile.seperateChannelActivityOption then
			if self.db.profile.groupChannelActive[num][8] then
				SendChatMessage(self.db.profile.reply[num],"YELL")
			end
		else
			SendChatMessage(self.db.profile.reply[num],"YELL")
		end
	elseif event == "CHAT_MSG_GUILD" and shortSource ~= UnitName("player") then
		if self.db.profile.seperateChannelActivityOption then 
			if self.db.profile.groupChannelActive[num][6] then
				SendChatMessage(self.db.profile.reply[num],"GUILD")
			end			
		else
			SendChatMessage(self.db.profile.reply[num],"GUILD")
		end
	elseif event == "CHAT_MSG_OFFICER" and shortSource ~= UnitName("player") then
		if self.db.profile.seperateChannelActivityOption then 
			if self.db.profile.groupChannelActive[num][7] then
				SendChatMessage(self.db.profile.reply[num],"OFFICER")
			end
		else
			SendChatMessage(self.db.profile.reply[num],"OFFICER")
		end
	elseif event == "CHAT_MSG_PARTY" and shortSource ~= UnitName("player") then
		if self.db.profile.seperateChannelActivityOption then 
			if self.db.profile.groupChannelActive[num][4] then
				SendChatMessage(self.db.profile.reply[num],"PARTY")
			end
		else
			SendChatMessage(self.db.profile.reply[num],"PARTY")
		end	
	elseif event == "CHAT_MSG_PARTY_LEADER" and shortSource ~= UnitName("player") then
		if self.db.profile.seperateChannelActivityOption then 
			if self.db.profile.groupChannelActive[num][4] then
				SendChatMessage(self.db.profile.reply[num],"PARTY")
			end
		else
			SendChatMessage(self.db.profile.reply[num],"PARTY")
		end	
	elseif event == "CHAT_MSG_RAID" and shortSource ~= UnitName("player") then
		if self.db.profile.seperateChannelActivityOption then 
			if self.db.profile.groupChannelActive[num][5] then
				SendChatMessage(self.db.profile.reply[num],"RAID")
			end
		else
			SendChatMessage(self.db.profile.reply[num],"RAID")
		end	
	elseif event == "CHAT_MSG_RAID_LEADER" and shortSource ~= UnitName("player") then
		if self.db.profile.seperateChannelActivityOption then 
			if self.db.profile.groupChannelActive[num][5] then
				SendChatMessage(self.db.profile.reply[num],"RAID")
			end
		else
			SendChatMessage(self.db.profile.reply[num],"RAID")
		end	
	elseif event == "CHAT_MSG_RAID_WARNING" and shortSource ~= UnitName("player") then
		if self.db.profile.seperateChannelActivityOption then 
			if self.db.profile.groupChannelActive[num][5] then
				SendChatMessage(self.db.profile.reply[num],"RAID")
			end
		else
			SendChatMessage(self.db.profile.reply[num],"RAID")
		end
	elseif event == "CHAT_MSG_WHISPER" and shortSource ~= UnitName("player") then
		if self.db.profile.seperateChannelActivityOption then 
			if self.db.profile.groupChannelActive[num][2] then
				SendChatMessage(self.db.profile.reply[num],"WHISPER", nil, fullSource)
			end
		else
			SendChatMessage(self.db.profile.reply[num],"WHISPER", nil, fullSource)
		end
	elseif event == "CHAT_MSG_BN_WHISPER" then
		if not BNIsSelf(pID) then
			if self.db.profile.seperateChannelActivityOption then 
				if self.db.profile.groupChannelActive[num][3] then
					C_Timer.After(.1, function() BNSendWhisper(pID,self.db.profile.reply[num]) end)
				end
			else
				C_Timer.After(.1, function() BNSendWhisper(pID,self.db.profile.reply[num]) end)
			end
		end
	end
end

CookyChatHandler:SetScript("OnEvent", function(_,event,message,fullSource,_,_,shortSource,_,_,_,_,_,_,_,pID)
	if (CookyChat:OnWhitelist(event, shortSource) or not CookyChat:OnBlacklist(event, shortSource)) then
		CookyChat:FindTrigger(message, event, shortSource, fullSource, pID)
	end
end)